@@ -21,11 +21,11 @@
 # SOFTWARE.
 import logging
 import threading
-from typing import List
-from typing import Optional
+from typing import Dict
 
+from squeaknode.core.squeak_core import SqueakCore
 from squeaknode.core.twitter_account_entry import TwitterAccountEntry
-from squeaknode.node.squeak_controller import SqueakController
+from squeaknode.node.squeak_store import SqueakStore
 from squeaknode.twitter.twitter_stream import TwitterStream
 
 
@@ -36,31 +36,46 @@ class TwitterForwarder:
 
     def __init__(
             self,
+            squeak_store: SqueakStore,
+            squeak_core: SqueakCore,
             retry_s: int,
     ):
+        self.squeak_store = squeak_store
+        self.squeak_core = squeak_core
         self.retry_s = retry_s
         self.lock = threading.Lock()
-        self.current_task: Optional[TwitterForwarderTask] = None
+        self.current_tasks: Dict[str, TwitterForwarderTask] = {}
 
-    def start_processing(self, squeak_controller: SqueakController):
+    def start_processing(self):
         with self.lock:
-            if self.current_task is not None:
-                self.current_task.stop_processing()
-            self.current_task = TwitterForwarderTask(
-                squeak_controller,
-                self.retry_s,
-            )
-            self.current_task.start_processing()
+            # Stop existing running tasks.
+            for handle, task in list(self.current_tasks.items()):
+                task.stop_processing()
+                del self.current_tasks[handle]
+
+            # Start new tasks.
+            for account in self.squeak_store.get_twitter_accounts():
+                task = TwitterForwarderTask(
+                    self.squeak_store,
+                    self.squeak_core,
+                    account,
+                    self.retry_s,
+                )
+                task.start_processing()
+                self.current_tasks[account.handle] = task
 
     def stop_processing(self):
         with self.lock:
-            if self.current_task is not None:
-                self.current_task.stop_processing()
+            for handle, task in list(self.current_tasks.items()):
+                task.stop_processing()
+                del self.current_tasks[handle]
 
     def is_processing(self) -> bool:
         with self.lock:
-            if self.current_task is not None:
-                return self.current_task.is_processing()
+            # if self.current_task is not None:
+            #     return self.current_task.is_processing()
+            # return False
+            # TODO:
             return False
 
 
@@ -68,10 +83,14 @@ class TwitterForwarderTask:
 
     def __init__(
         self,
-        squeak_controller: SqueakController,
+        squeak_store: SqueakStore,
+        squeak_core: SqueakCore,
+        twitter_account: TwitterAccountEntry,
         retry_s: int,
     ):
-        self.squeak_controller = squeak_controller
+        self.squeak_store = squeak_store
+        self.squeak_core = squeak_core
+        self.twitter_account = twitter_account
         self.retry_s = retry_s
         self.stopped = threading.Event()
         self.tweet_stream = None
@@ -84,15 +103,18 @@ class TwitterForwarderTask:
             daemon=True,
         ).start()
 
-    def setup_stream(self, bearer_token, handles):
-        logger.info("Starting Twitter stream with bearer token: {} and twitter handles: {}".format(
-            bearer_token,
-            handles,
+    def setup_stream(self):
+        logger.info("Starting Twitter stream with bearer token: {} and twitter handle: {}".format(
+            self.twitter_account.bearer_token,
+            self.twitter_account.handle,
         ))
         with self.lock:
             if self.stopped.is_set():
                 return
-            twitter_stream = TwitterStream(bearer_token, handles)
+            twitter_stream = TwitterStream(
+                self.twitter_account.bearer_token,
+                [self.twitter_account.handle],
+            )
             self.tweet_stream = twitter_stream.get_tweets()
 
     def stop_processing(self):
@@ -111,13 +133,7 @@ class TwitterForwarderTask:
 
         while not self.stopped.is_set():
             try:
-                bearer_token = self.get_bearer_token()
-                handles = self.get_twitter_handles()
-                if not bearer_token:
-                    return
-                if not handles:
-                    return
-                self.setup_stream(bearer_token, handles)
+                self.setup_stream()
                 for tweet in self.tweet_stream.result_stream:
                     self.handle_tweet(tweet)
             # TODO: use more specific error.
@@ -131,31 +147,37 @@ class TwitterForwarderTask:
                 self.stopped.wait(wait_s)
                 wait_s *= 2
 
-    def get_bearer_token(self) -> str:
-        return self.squeak_controller.get_twitter_bearer_token() or ''
-
-    def get_twitter_handles(self) -> List[str]:
-        twitter_accounts = self.squeak_controller.get_twitter_accounts()
-        handles = [account.handle for account in twitter_accounts]
-        return handles
-
-    def is_tweet_a_match(self, tweet: dict, account: TwitterAccountEntry) -> bool:
+    def is_tweet_a_match(self, tweet: dict) -> bool:
         for rule in tweet['matching_rules']:
-            if rule['tag'] == account.handle:
+            if rule['tag'] == self.twitter_account.handle:
                 return True
         return False
 
-    def forward_tweet(self, tweet: dict, account: TwitterAccountEntry) -> None:
-        self.squeak_controller.make_squeak(
-            profile_id=account.profile_id,
+    def forward_tweet(self, tweet: dict) -> None:
+        self.make_squeak(
+            profile_id=self.twitter_account.profile_id,
             content_str=tweet['data']['text'],
-            replyto_hash=None,
         )
 
     def handle_tweet(self, tweet: dict):
         logger.info(
             "Got tweet: {}".format(tweet))
-        twitter_accounts = self.squeak_controller.get_twitter_accounts()
-        for account in twitter_accounts:
-            if self.is_tweet_a_match(tweet, account):
-                self.forward_tweet(tweet, account)
+        if self.is_tweet_a_match(tweet):
+            self.forward_tweet(tweet)
+
+    def make_squeak(self, profile_id: int, content_str: str):
+        squeak_profile = self.squeak_store.get_squeak_profile(profile_id)
+        if squeak_profile is None:
+            raise Exception("Profile with id {} not found.".format(
+                profile_id,
+            ))
+        squeak, secret_key = self.squeak_core.make_squeak(
+            squeak_profile,
+            content_str,
+        )
+        inserted_squeak_hash = self.squeak_store.save_squeak(
+            squeak,
+        )
+        if inserted_squeak_hash is None:
+            return None
+        self.squeak_store.save_secret_key(inserted_squeak_hash, secret_key)
